//
//  EnhancedTableViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 11/10/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import SafariServices

class EnhancedTableViewController: UITableViewController {

    var prevScrollToTopOffset: CGPoint? = nil

    private var topOffset: CGPoint {
        // when scrolled to top, the content offset is negative the height of the UI above the scroll view (i.e. the nav and status bars)
        let windowScene = view.window!.windowScene!
        let barOffset = -1 * (navigationController!.navigationBar.frame.height + windowScene.statusBarManager!.statusBarFrame.height)
        // add one so it's not technically all the way at the top, and scrollViewWShouldScrollToTop is still called to trigger undo
        return CGPoint(x: 0, y: barOffset + 1)
    }

    override func scrollViewShouldScrollToTop(_ scrollView: UIScrollView) -> Bool {
        if let offset = prevScrollToTopOffset {
            tableView.setContentOffset(offset, animated: true)
            prevScrollToTopOffset = nil
        } else {
            prevScrollToTopOffset = tableView.contentOffset
            tableView.setContentOffset(topOffset, animated: true)
        }
        return false
    }

    override func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        prevScrollToTopOffset = nil
    }
    
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let cell = tableView.cellForRow(at: indexPath) as? SelectableTableViewCell {
            cell.didSelectCell()
        }
    }

}

extension EnhancedTableViewController {
    
    override func tableView(_ tableView: UITableView, contextMenuConfigurationForRowAt indexPath: IndexPath, point: CGPoint) -> UIContextMenuConfiguration? {
        if let cell = tableView.cellForRow(at: indexPath) as? UITableViewCell & MenuPreviewProvider {
            let cellLocation = cell.convert(point, from: tableView)
            guard let (previewProvider, actionsProvider) = cell.getPreviewProviders(for: cellLocation, sourceViewController: self) else {
                return nil
            }
            let actionProvider: UIContextMenuActionProvider = { (_) in
                let suggested = self.getSuggestedContextMenuActions(tableView: tableView, indexPath: indexPath, point: point)
                return UIMenu(title: "", image: nil, identifier: nil, options: [], children: suggested + actionsProvider())
            }
            return UIContextMenuConfiguration(identifier: nil, previewProvider: previewProvider, actionProvider: actionProvider)
        } else {
            return nil
        }
    }
    
    // todo: replace this with the UIKit suggested actions, if possible
    @objc open func getSuggestedContextMenuActions(tableView: UITableView, indexPath: IndexPath, point: CGPoint) -> [UIAction] {
        return []
    }
    
    override func tableView(_ tableView: UITableView, willPerformPreviewActionForMenuWith configuration: UIContextMenuConfiguration, animator: UIContextMenuInteractionCommitAnimating) {
        if let viewController = animator.previewViewController {
            animator.preferredCommitStyle = .pop
            animator.addCompletion {
                if viewController is LargeImageViewController || viewController is GalleryViewController || viewController is SFSafariViewController {
                    self.present(viewController, animated: true)
                } else {
                    self.show(viewController, sender: nil)
                }
            }
        }
    }
    
}
